// zControl.cc
// Classes and functions required for User-configurable Text Objects 
// By Peter Rorlick
// For more information, see ConfigTx.HTM
********************************************************************
class zBaseText(parentObj) of Text(parentObj) custom
endclass
********************************************************************
class zText(parentObj) of zBaseText(parentObj) custom

   with( this )
     OnRightMouseDown = {; do configTx.wfm with true, this.form, this}
     text = 'ZText'
     wrap = false
     height = 0.8
     width = 20
   endWith

   Function Init
      if file('ConfigTx.dbf')
        // Look up Form.ClassName+'.'+this.name, and set properties if found.
        if type('_app.qConfigTx')=='U'
          // Create and activate a query on ConfigTx.dbf (once only):
          _app.qConfigTx = new Query()
          _app.qConfigTx.SQL = 'select * from ConfigTx'
          _app.qConfigTx.Active = true
          _app.qConfigTx.Rowset.IndexName = 'ObjName'
        endif
        *
        // To get the full object name for objects that reside on
        // containers/notebooks, we parse the ownership tree
        // (Thanks to George Burt for this technique):
        local mObject
        private mTemp1
        Mobject  = this.name
        Mtemp1 = this.parent
        do while type("Mtemp1.parent") # 'U'
          Mobject = Mtemp1.name+"."+Mobject
          Mtemp1 = Mtemp1.parent
        enddo
        *
        if _app.qConfigTx.Rowset.FindKey(Form.ClassName+'.'+mObject)
          // We've found a set of user preferences for this text object.
          // Let's apply those preferences:
          this.Text        = Trim(_app.qConfigTx.Rowset.Fields['pText'].Value)
          this.Top         = _app.qConfigTx.Rowset.Fields['pTop'].Value
          this.Left        = _app.qConfigTx.Rowset.Fields['pLeft'].Value
          this.Width       = _app.qConfigTx.Rowset.Fields['pWidth'].Value
          this.Height      = _app.qConfigTx.Rowset.Fields['pHeight'].Value
          this.ColorNormal = Trim(_app.qConfigTx.Rowset.Fields['pColorNorm'].Value)
          this.Wrap        = _app.qConfigTx.Rowset.Fields['pWrap'].Value
          this.Function    = Trim(_app.qConfigTx.Rowset.Fields['pFunction'].Value)
          this.Picture     = Trim(_app.qConfigTx.Rowset.Fields['pPicture'].Value)
          this.FontName    = Trim(_app.qConfigTx.Rowset.Fields['pFontName'].Value)
          this.FontSize    = _app.qConfigTx.Rowset.Fields['pFontSize'].Value
          this.FontBold    = _app.qConfigTx.Rowset.Fields['pFontBold'].Value
          this.FontItalic  = _app.qConfigTx.Rowset.Fields['pFontItal'].Value
          this.Border      = _app.qConfigTx.Rowset.Fields['pBorder'].Value
          if _app.qConfigTx.Rowset.Fields['pBorder'].Value
            this.BorderStyle = 7   // Client style
          endif
        endif
      endif    //if file('ConfigTx.dbf')
      *
   return

endclass
********************************************************************
// Pete's zForm (VdB7 base form) class
// Thanks to Romain and Bowen for their ideas.
//
//  Things happen in the following order, each time you call form.open() or form.readModal():
//   1. Form.init() method executes.
//   2. Init() methods of objects on the form execute, one by one. (But only the first time the form is opened.)
//   3. Form.AfterControlInits() method executes.
//   4. The form becomes visible.
//   5. Form.OnOpen() method executes.
//   6. OnOPen() methods of objects on the form execute, one by one.
//
//  Bear in mind that the CONSTRUCTOR section of your form fires ONLY when you do
//    <something> = new WhateverForm()
//
class zForm of Form custom

  with (this)
     top = 3
     height = 15
     width = 50
     Left = 10
     text = 'zForm'
     AutoCenter = true
  endwith
  this.FirstTime = true

  Proc Open
    class::BeforeOpen()
  return super::open()

  Proc ReadModal
    if class::BeforeOpen()
      return super::ReadModal()
    endif
  return

  Func BeforeOpen
    form.Init()
    if form.FirstTime
      CLASS::RunControlInits(Form)   // Only runs the first time the form is opened.
    endif
    form.AfterControlInits()
    form.FirstTime := false
  return true   // Allow the form to open

  Proc Init
    // Over-ride this procedure in forms that are inherited from this class,
    // to do any stuff that needs to happen when the form opens.
    // Use an OnOpen event handler only for the stuff that must be done AFTER
    // a form opens - i.e., stuff that requires a form window handle.
  return

  Proc AfterControlInits
    // Over-ride this procedure in forms that are inherited from this class,
    // to do any stuff that needs to happen when the form opens but AFTER
    // the Init methods of its controls have been run.
  return

  Proc RunControlInits(InitialObject)
    // Run the Init methods of any controls that have such methods.
    // (This is often faster than running OnOpen events, where the
    // form is already open.)
    // For example, your form's Init method could have something like this:
    //    Form.EntryField1.Init = CLASS::DoWhatever  // function pointer
    //    Form.EntryField2.Init = {; SomeClass::SomeMethod() ; CLASS::DoWhatever() }
    // In some cases, this can give you better performance than using the OnOpen
    // events of the controls. The reason is that when the Init methods are fired
    // the form is not yet open, so there are no GUI controls to refresh.
    private o
    o = InitialObject.First
    do
      if Type('o.Init') $ 'FP/CB'   // If this control object has an Init method...
        o.Init()
      endif
      // The next line tests whether o is a container or notebook (but not a
      // form, because forms have a UseTablePopUp property) that contains
      // other objects...
      If Type('o.First')=='O' and type('o.UseTablePopUp')=='U'
        CLASS::RunControlInits(o)   // recurse through the objects of the container
      EndIf
      o = o.before
    until o.name == InitialObject.First.name
  return

  Procedure Repaint   // Call this method to refresh the form. More thorough than Form.Refresh().
    if type("UpdateWindow") # "FP"
      extern cLogical UpdateWindow(cHandle) user32
    endif
    if argcount()=1
       UpdateWindow(argVector(1))
    else
       UpdateWindow(form.hwnd)
    endif
  Return

endClass
*********************************************************************
Class zEditor(FormObj) OF Editor(FormObj) Custom   // Zoomable editor

  this.BorderStyle = 7
  this.ScrollBar   = 1
  this.ColorNormal='N/W+'
  this.PopUpEnable := false   // Prevent glitch with modal forms
  this.cuaTab := false

  this.lZoomModal = true   // Can be overridden. This flag is passed as (bModal)
                      // when ZoomEdit.wfm is called. If you want to enable
                      // HTML formatting, set this to FALSE in your form.

  Procedure Init
   // Define Zoom button:
   PRIV c
   oParentOfEditor = this.Parent
   c ='oParentOfEditor.Z' + TRIM(This.Name) + ' = new PUSHBUTTON(oParentOfEditor)'
   &c
   c = 'oParentOfEditor.Z' + TRIM(This.Name)
   this.oZoomButton = &c
   With (&c)
     Top        = This.Top+0.15
     Left       = This.Left+This.Width-(2.5+2.9)
     Height     = 0.9
     Width      = 2.9
     Text       = ""
     Group      = true
     OnClick    = CLASS::ZoomEdit
     PageNo     = This.PageNo
     Enabled    = This.Enabled
     Visible    = This.Visible
     UpBitmap   = "RESOURCE #2001"
     TabStop    = false
     SpeedBar   = true
     SpeedTip   = "Zoom in"
   EndWith
   priv d
   d = &c    // d is now the zoom button
   d.oOwner = this   // reference to the editor that "owns" the zoom button.
   d.OnFormResize = {; this.Move(this.oOwner.Left+This.oOwner.Width-(2.5+2.9), this.oOwner.Top+0.15) }
  return

  Proc ZoomEdit             // The OnClick event of the Zoom button.
    oEditor = This.oOwner
    _app.oEditor = oEditor
    do ZoomEdit.WFM with this.oOwner.lZoomModal
  return

endClass
**********************************************************************
CLASS zSpinBox(FormObj) OF SpinBox(FormObj) Custom
  with( this )
    borderStyle = 7
    ColorNormal='N/W+'
    ColorHighlight = 'N/RG+'
    Height = 1
    FontSize = 10
    function = 'J'  // Right-align
  endWith
ENDCLASS
****************************************************************************
class zCheckBox(parentObj) of CHECKBOX(parentObj) custom
   with (this)
      left = 5
      top = 2
   endwith
   this.BaseClass_Name = 'CHECKBOX'
endclass
**********************************************************************
CLASS zEntryField(FormObj) OF ENTRYFIELD(FormObj) Custom
  with (this)
    onGotFocus = {; this.keyboard( "{Home}" )}
    BorderStyle = 7
    ColorHighlight = 'N/RG+'  // yellow background when in focus.
    ColorNormal = 'N/W+'
    Height = 1
    SelectAll = false   // So that selected strings are immediately visible even if they have lots of trailing spaces.
  endwith
ENDCLASS
****************************************************************************
class zContainer(parentObj) of CONTAINER(parentObj) custom
   with (this)
      width = 20
      height = 10
   endwith
   this.BaseClass_Name = 'CONTAINER'

endclass
************************************************************************
class FontPicker(parentObj) of ZCONTAINER(parentObj) custom
   with (this)
      left = 6.2857
      top = 1.0909
      width = 52.2857
      height = 3.4091
      metric = 0        // Chars
   endwith


   this.TEXT1 = new ZBASETEXT(this)
   with (this.TEXT1)
      height = 1
      left = 1.7143
      top = 0.5909
      width = 5.8571
      wrap = false
      alignVertical = 1        // Middle
      text = "Font"
   endwith


   this.FontName = new ZENTRYFIELD(this)
   with (this.FontName)
      height = 1
      left = 8.4286
      top = 0.5909
      width = 41.1429
      value = "                         "
   endwith


   this.FontBtn = new sBUTTON(this)
   with (this.FontBtn)
      onClick = class::FONTBTN_onClick
      height = 0.8636
      left = 46.1429
      top = 0.6818
      width = 3.1429
      text = ""
      upBitmap = "RESOURCE #148"
      value = false
   endwith


   this.TEXT2 = new ZBASETEXT(this)
   with (this.TEXT2)
      height = 1
      left = 8.4286
      top = 1.7727
      width = 4.4286
      wrap = false
      alignVertical = 1        // Middle
      text = "Size"
   endwith


   this.FontSize = new ZSPINBOX(this)
   with (this.FontSize)
      height = 1
      left = 13.7143
      top = 1.8182
      width = 9.5714
      rangeMax = 100
      rangeMin = 1
      value = 1
   endwith


   this.FontBold = new ZCHECKBOX(this)
   with (this.FontBold)
      height = 1.0909
      left = 29.2857
      top = 1.7727
      width = 7
      text = "&Bold"
      value = false
   endwith


   this.FontItalic = new ZCHECKBOX(this)
   with (this.FontItalic)
      height = 1.0909
      left = 42
      top = 1.7727
      width = 7.1429
      text = "&Italic"
      value = false
   endwith


   function FONTBTN_onClick
      local cFont
      cFont = getfont()
      if not isblank(cFont)
        this.parent.FontName.Value = left(cFont,at(',',cFont)-1)
        cFont = subs(cFont,at(',',cFont)+1)   // now it's nSize,[B][I],cSwiss
        this.parent.FontSize.Value = val(left(cFont,at(',',cFont)-1))
        cFont = subs(cFont,at(',',cFont)+1)   // now it's [B][I],cSwiss
        this.parent.FontBold.Value = (left(cFont,1)=='B')
        this.parent.FontItalic.Value = ('I'$left(cFont,2))
        Form.Repaint()
        if not empty( this.parent.FontName.onChange )
          this.parent.FontName.onChange()
        endif
      endif
  return

endclass
************************************************************************
class ColorPicker(parentObj) of zEntryField(parentObj) custom

   function Init
     // Define a button to the right of the entryfield, to invoke 
     // a color-picker dialog:
     this.colorBtn = new sBUTTON(this.parent)
     with (this.colorBtn)
        onClick = class::colorBtn_ONCLICK
        Top = This.Top + .1
        Left = This.Left+This.Width-3.1429 - .3
        Height = This.Height - .15
        Width = 3.1429
        PageNo = This.PageNo
        upBitmap = "RESOURCE #148"
     endwith
     this.colorBtn.oOwner = this  // Create a reference to the entryfield.
   return

   function COLORBTN_onClick
     local cColors
     cColors = Colors() // colors.prg
     if not isblank(cColors)
       this.oOwner.value := cColors
     endif
   return

endClass


